/**
 * File: DynamicInvocation.cs
 * Article: C# Reflection and Dynamic Method Invocation
 * Copyright (c) 2000, Gopalan Suresh Raj. All Rights Reserved.
 * Compile with: csc DynamicInvocation.cs
 * Run As: DynamicInvocation <library | Application...>
 *         eg., DynamicInvocation ImplementationOne.DLL ReflectMethods.EXE
 */
 
 using System;
 using System.Reflection;
 using com.icommware;
 
 class DynamicInvocation {
 
  public static void invokeProcess (string assemblyName) {
   Console.WriteLine ("Loading {0} into memory...", assemblyName);
   Assembly assembly = Assembly.LoadFrom (assemblyName);
   
   // Walk through each type in the assembly
   foreach (Type type in assembly.GetTypes ()) {
    // Pick up a class
    if (type.IsClass == true) {
     Console.WriteLine ("...Found Class : {0}", type.FullName);
     
     // If it does not implement the IBase Interface, skip it
     if (type.GetInterface ("com.icommware.IBase") == null) {
      continue;
     }
     
     // If however, it does implement the IBase Interface,
     // create an instance of the object
     object ibaseObject = Activator.CreateInstance (type);
     
     // Create the parameter list
     object[] arguments = new object [] {10, 17.11};
     object result;
     // Dynamically Invoke the Object
     Console.WriteLine ("......Dynamically Invoking compute() on {0} Class", type.FullName);
     result = type.InvokeMember ("compute", 
                                 BindingFlags.Default | BindingFlags.InvokeMethod,
                                 null,
                                 ibaseObject,
                                 arguments);
                                
     Console.WriteLine ("......Result is: {0}", result);     
    }
   }
  }
  
  public static void Main (string[] args) {
   foreach (string argument in args) {
    invokeProcess (argument);
   }
  }
  
 }

