/**
 * File: ReflectMethods.cs
 * Article: C# Reflection and Dynamic Method Invocation
 * Copyright (c) 2000, Gopalan Suresh Raj. All Rights Reserved.
 * Compile with: csc ReflectMethods
 */

using System;
using System.Reflection;

namespace com.icommware {

 /**
  * An enumeration of Account Types
  */
 enum AccountEnumeration {
  CHECKING = 1,
  SAVINGS = 2,
 }

 /**
  * A Class that implements the IBase interface
  */
 class ImplementationTwo : IBase {
  string remark = "Remark of ImplementationTwo";
  static int DELTAFACTOR = 100;

  ImplementationTwo () {
   Console.WriteLine ("Invoking ImplementationTwo ...");
  }
  
  public string execute (string data) {
   remark = data;
   Console.WriteLine ("ImplementationTwo's execute() remark is : {0}", remark);
   return remark;
  }
  
  public double compute (int param1, double param2) {
   Console.WriteLine ("ImplementationTwo's compute() method invoked...");
   return param1*param2*DELTAFACTOR;
  }
 }
 
 /**
  * A generic Structure
  */
 struct StructTwo {
  double x;
  int y;
  
  double assign () {
   x = 8.2;
   y = 10;
   return x*y;
  }
 }
}

/**
 * Develop a class that can Reflect all the Types available in an Assembly
 */
class ReflectMethods {

 public static void Main (string[] args) {
 
  // retrieve information about AccountEnumeration 
  Console.WriteLine ("Names and Values of the com.icommware.AccountEnumeration are:");
  Type type = typeof (com.icommware.AccountEnumeration );
  
  // Create an instance of AccountEnumeration in Memory
  object accountEnum = Activator.CreateInstance (type);
  
  // List the names and values available in the AccountEnumeration  enum
  foreach (FieldInfo fieldInfo in type.GetFields (BindingFlags.LookupAll)) {
   object value = fieldInfo.GetValue (accountEnum);
   Console.WriteLine ("{0} = {1}", fieldInfo, value);
  }
  Console.WriteLine ("_____");
  
  // retrieve information about ImplementationTwo Class
  Console.WriteLine ("The Members of the ImplementationTwo Class are:");
  type = typeof (com.icommware.ImplementationTwo);
  
  // List the members of the ImplementationTwo Class
  foreach (MemberInfo memberInfo in type.GetFields (BindingFlags.LookupAll)) {
   Console.WriteLine ("{0}", memberInfo);
  }
  Console.WriteLine ("_____");
  
  // Walk through the methods of the ImplementationTwo Class
  Console.WriteLine ("The Methods of the ImplementationTwo Class are:");
  foreach (MethodInfo methodInfo in type.GetMethods (BindingFlags.LookupAll)) {
   // Print the Method Signature
   Console.WriteLine ("{0}", methodInfo);
   // Walk through the method parameters
   foreach (ParameterInfo parameterInfo in methodInfo.GetParameters ()) {
    // Print the Parameter Type and the Parameter Name
    Console.WriteLine (" Param: {0} {1}", parameterInfo.ParameterType, parameterInfo.Name);
   }
   Console.WriteLine ("_____");
  }
  
  
  // retrieve information about StructTwo Class
  Console.WriteLine ("The Members of the StructTwo Class are:");
  type = typeof (com.icommware.StructTwo);
  
  // List the members of the StructTwo Class
  foreach (MemberInfo memberInfo in type.GetFields (BindingFlags.LookupAll)) {
   Console.WriteLine ("{0}", memberInfo);
  }
  Console.WriteLine ("_____");
  // Walk through the methods of the StructTwo Class
  Console.WriteLine ("The Methods of the StructTwo Class are:");
  foreach (MethodInfo methodInfo in type.GetMethods (BindingFlags.LookupAll)) {
   // Print the Method Signature
   Console.WriteLine ("{0}", methodInfo);
   // Walk through the method parameters
   foreach (ParameterInfo parameterInfo in methodInfo.GetParameters ()) {
    // Print the Parameter Type and the Parameter Name
    Console.WriteLine (" Param: {0} {1}", parameterInfo.ParameterType, parameterInfo.Name);
   }
   Console.WriteLine ("_____");
  }
 }
}